import {
  select,
  csv,
  scaleLinear,
  extent,
  axisBottom,
  axisLeft,
  format,
} from 'd3';
import { dropdownMenu } from './dropdownMenu';
import { horizontalChart } from './horizontalChart';
import { scatterPlot } from './scatterPlot';

const svg = select('svg');

const width = +svg.attr('width');
const height = +svg.attr('height');

let visualization = 'main';

const onVisualizationClicked = (column) => {
  visualization = column;
  render(visualization);
};

//main page
let data;
let filteredData;
let typeCounts = {};
let types;
const allTypes = {
  normal: '#AAB09F',
  fighting: '#C22E28',
  flying: '#7DA6DE',
  poison: '#A33EA1',
  ground: '#927D44',
  rock: '#B2A061',
  bug: '#94BC4A',
  ghost: '#846AB6',
  steel: '#89A1B0',
  fire: '#EE8130',
  water: '#6390F0',
  grass: '#7AC74C',
  electric: '#E5C531',
  psychic: '#E5709B',
  ice: '#96D9D6',
  dragon: '#6A7BAF',
  dark: '#736C75',
  fairy: '#E397D1',
};
let selectedType = null;
const topNumberInput = document.getElementById('topNumber');
topNumberInput.addEventListener('input', function () {
  filteredData = data.slice(0, topNumberInput.value);
  //console.log(filteredData);
  types.forEach((type) => {
    if (type != 'none') {
      typeCounts[type] = filteredData.filter(
        (d) => d['Type 1'] === type || d['Type 2'] === type,
      ).length;
    }
  });
  svg.selectAll('*').remove();
  selectedType = null;
  render(visualization);
});

//fast moves
let fastMovesData;
let chargeMovesData;

const render = (visualization) => {
  svg.selectAll('*').remove();
  // Create a table for types and type effectiveness
  switch (visualization) {
    case 'main':
      selectedType = null;
      const table = svg
        .append('foreignObject')
        .attr('transform', `translate(400,0)`)
        .attr('width', 700) // Set the width of the table
        .attr('height', height)
        .append('xhtml:table')
        .style('border-collapse', 'collapse');

      // Add header row
      const headerRow = table.append('thead').append('tr');
      headerRow.append('th').text('Types\nDefending↓');
      types.forEach((type) => {
        headerRow
          .append('th')
          .text(type.substring(0, 3).toUpperCase())
          .style('background-color', allTypes[type]) // Set background color
          .style('color', 'white');
      });

      // Add rows for type effectiveness
      const tbody = table.append('tbody');
      types.forEach((type1) => {
        const row = tbody
          .append('tr')
          .attr('class', `row-${type1}`);
        row
          .append('td')
          .text(type1)
          .style('background-color', allTypes[type1]) // Set background color
          .style('color', 'white');

        types.forEach((type2) => {
          const effectiveness = getTypeEffectiveness(
            type1,
            type2,
          );
          const cell = row
            .append('td')
            .attr(
              'class',
              effectiveness != 1 ? 'filled' : 'blank',
            )
            .style(
              'background-color',
              effectiveness > 1
                ? 'green'
                : effectiveness < 0.4
                  ? 'red'
                  : effectiveness < 0.7
                    ? 'orange'
                    : 'transparent',
            )
            .on('mouseover', function () {
              const [x, y] = d3.mouse(this);
              // Show tooltip dynamically on mouseover
              const tooltip = select(this)
                .append('div')
                .classed('tooltip', true);
              tooltip
                .html(
                  `${type2} ->${type1} = ${
                    effectiveness > 1
                      ? 'super effective'
                      : effectiveness < 0.4
                        ? 'not effective'
                        : effectiveness < 0.7
                          ? 'not very effective'
                          : 'normal effectiveness'
                  }`,
                )
                .style(
                  'transform',
                  `translate(${x}px, ${y}px)`,
                );
            })
            .on('mouseout', function () {
              // Remove the tooltip on mouseout
              select(this).select('.tooltip').remove();
            });
        });
      });

      svg
        .append('text')
        .attr('class', 'selected-type')
        .attr('x', 400)
        .attr('y', height - 160)
        .attr('text-anchor', 'left')
        .text(`Selected Types: `);

      // Call horizontalChart function
      svg.call(horizontalChart, {
        margin: {
          top: 0,
          right: 800,
          bottom: 100,
          left: 70,
        },
        width,
        height,
        total: topNumberInput.value,
        data: typeCounts,
        attributes: allTypes,
        xAxisLabel: 'Count',
        yAxisLabel: 'Types',
        onTypeSelect: (type) => {
          selectedType = type;
          updateTableStyles();
        },
      });
      updateTableStyles();
      break;
    case 'fast moves':
      const pokemonMovesMap = new Map();
      filteredData.forEach((pokemon, index) => {
        const fastMove = pokemon['Fast Move'];

        // Check if the move already exists in the map
        if (pokemonMovesMap.has(fastMove)) {
          // If it exists, add the current Pokemon to the list
          const pokemonList = pokemonMovesMap.get(fastMove);
          pokemonList.push({
            name: pokemon['Pokemon'],
            rank: index + 1,
          });
        } else {
          // If it doesn't exist, create a new list with the current Pokemon
          pokemonMovesMap.set(fastMove, [
            {
              name: pokemon['Pokemon'],
              rank: index + 1,
            },
          ]);
        }
      });

      console.log(pokemonMovesMap);

      svg.call(scatterPlot, {
        margin: {
          top: 30,
          right: 100,
          bottom: 70,
          left: 100,
        },
        width,
        height,
        data: fastMovesData,
        xValue: (d) => d['DPT'],
        xAxisLabel: 'damage per turn (DPT)',
        yValue: (d) => d['EPT'],
        yAxisLabel: 'energy per turn (EPT)',
        colorLegendLabel: 'Types',
        colorValue: (d) => d.Types,
        colorRange: allTypes,
        addition: pokemonMovesMap,
        number: filteredData.length,
        fastOrCharge: 'fast',
      });
      break;
    case 'charge moves':
      const pokemonChargeMovesMap = new Map();
      filteredData.forEach((pokemon, index) => {
        const chargeMove = pokemon['Charged Move 1'];
        const chargeMove2 = pokemon['Charged Move 2'];
        // Check if the move already exists in the map
        if (pokemonChargeMovesMap.has(chargeMove)) {
          // If it exists, add the current Pokemon to the list
          const pokemonList =
            pokemonChargeMovesMap.get(chargeMove);
          pokemonList.push({
            name: pokemon['Pokemon'],
            rank: index + 1,
          });
        } else {
          // If it doesn't exist, create a new list with the current Pokemon
          pokemonChargeMovesMap.set(chargeMove, [
            {
              name: pokemon['Pokemon'],
              rank: index + 1,
            },
          ]);
        }
        if (pokemonChargeMovesMap.has(chargeMove2)) {
          // If it exists, add the current Pokemon to the list
          const pokemonList =
            pokemonChargeMovesMap.get(chargeMove2);
          pokemonList.push({
            name: pokemon['Pokemon'],
            rank: index + 1,
          });
        } else {
          // If it doesn't exist, create a new list with the current Pokemon
          pokemonChargeMovesMap.set(chargeMove2, [
            {
              name: pokemon['Pokemon'],
              rank: index + 1,
            },
          ]);
        }
      });
      console.log(pokemonChargeMovesMap);
      svg.call(scatterPlot, {
        margin: {
          top: 30,
          right: 100,
          bottom: 70,
          left: 100,
        },
        width,
        height,
        data: chargeMovesData,
        xValue: (d) => d['Base Power'],
        xAxisLabel: 'Base Power',
        yValue: (d) => d['Energy'],
        yAxisLabel: 'Energy',
        colorLegendLabel: 'Types',
        colorValue: (d) => d.Types,
        colorRange: allTypes,
        addition: pokemonChargeMovesMap,
        number: filteredData.length,
        fastOrCharge: 'charge',
      });
      break;
  }
};

const updateTableStyles = () => {
  types.forEach((type) => {
    svg.selectAll(`.row-${type}`).style('opacity', 1);
  });
  //console.log(selectedType);
  // Update row styles based on the selected type
  if (!Array.isArray(selectedType)) {
    svg
      .selectAll(`.row-${selectedType}`)
      .selectAll('.blank')
      .style('background-color', '#8888FF')
      .style('opacity', 0.6);
    types.forEach((type) => {
      if ((selectedType != null) & (type != selectedType)) {
        svg.selectAll(`.row-${type}`).style('opacity', 0.3);
      }
    });
  } else {
    types.forEach((type) => {
      if (
        (selectedType.length != 0) &
        (type != selectedType[0]) &
        (type != selectedType[1])
      ) {
        svg.selectAll(`.row-${type}`).style('opacity', 0.3);
      }
    });
    svg.selectAll('.selected-type-text').remove();
    svg.selectAll('.effectiveness-text').remove();
    svg.selectAll('.effectiveness-content').remove();
    // Reset all rows to the default background color
    svg
      .selectAll('.blank')
      .style('background-color', 'white');

    // Set the background color for the selected rows
    selectedType.forEach((type, i) => {
      svg
        .selectAll(`.row-${type}`)
        .selectAll('.blank')
        .style('background-color', '#8888FF');
      svg
        .append('text')
        .attr('class', 'selected-type-text')
        .attr('x', 650 + 100 * i) // Adjust the x-coordinate as needed
        .attr('y', height - 160) // Adjust the y-coordinate as needed
        .attr('text-anchor', 'middle')
        .attr('fill', allTypes[type])
        .text(`${type}`);
    });

    if (selectedType.length >= 1) {
      let combinedEffectivenessMap;
      if (selectedType.length == 2) {
        combinedEffectivenessMap =
          calculateCombinedEffectiveness(
            selectedType[0],
            selectedType[1],
          );
      } else if (selectedType.length == 1) {
        combinedEffectivenessMap =
          calculateCombinedEffectiveness(
            selectedType[0],
            selectedType[0],
          );
      }

      // Create an object to store merged lines
      const weakness = {};
      const resistance = {};

      // Iterate through the map entries
      for (const [
        attacker,
        combinedEffectiveness,
      ] of combinedEffectivenessMap.entries()) {
        const formattedEffectiveness = format('.1f')(
          combinedEffectiveness,
        );
        if (formattedEffectiveness != 1) {
          // If the line already exists, append the defender to it
          if (formattedEffectiveness > 1) {
            if (weakness[formattedEffectiveness]) {
              weakness[formattedEffectiveness].push(
                attacker,
              );
            } else {
              // Otherwise, create a new line
              weakness[formattedEffectiveness] = [attacker];
            }
          } else {
            if (resistance[formattedEffectiveness]) {
              resistance[formattedEffectiveness].push(
                attacker,
              );
            } else {
              // Otherwise, create a new line
              resistance[formattedEffectiveness] = [
                attacker,
              ];
            }
          }
        }
      }

      // Create an array of lines for type effectiveness
      const weakLines = Object.entries(weakness).map(
        ([combinedEffectiveness, attackers]) =>
          `* ${combinedEffectiveness}: ${attackers.join(
            ', ',
          )}`,
      );

      const resistLines = Object.entries(resistance).map(
        ([combinedEffectiveness, attackers]) =>
          `* ${combinedEffectiveness}: ${attackers.join(
            ', ',
          )}`,
      );

      // Display the text with line breaks
      svg
        .append('text')
        .attr('class', 'effectiveness-text')
        .attr('x', 400)
        .attr('y', height - 140)
        .attr('text-anchor', 'left')
        .attr('fill', 'green')
        .text(`Weakness: `);
      svg
        .append('text')
        .attr('class', 'effectiveness-content')
        .attr('x', 400)
        .attr('y', height - 135)
        .attr('text-anchor', 'left')
        .selectAll('tspan')
        .data(weakLines)
        .enter()
        .append('tspan')
        .attr('x', 400)
        .attr('dy', 15) // Adjust the line height as needed
        .text((line) => line);

      svg
        .append('text')
        .attr('class', 'effectiveness-text')
        .attr('x', 750)
        .attr('y', height - 140)
        .attr('text-anchor', 'left')
        .attr('fill', 'red')
        .text(`Resistance: `);
      svg
        .append('text')
        .attr('class', 'effectiveness-content')
        .attr('x', 750)
        .attr('y', height - 135)
        .attr('text-anchor', 'left')
        .selectAll('tspan')
        .data(resistLines)
        .enter()
        .append('tspan')
        .attr('x', 750)
        .attr('dy', 15) // Adjust the line height as needed
        .text((line) => line);
    }
  }
};

function darkenColor(color, factor) {
  return d3.color(color).darker(factor).toString();
}

// Helper function to get type effectiveness (replace with your own logic)
const getTypeEffectiveness = (type1, type2) => {
  // Replace this logic with your own type effectiveness calculations
  // For example, you can use a data structure or function to determine effectiveness
  const typeEffectiveness = {};
  types.forEach((defender) => {
    typeEffectiveness[defender] = {};
    types.forEach((attacker) => {
      typeEffectiveness[defender][attacker] = 1.0;
    });
  });
  typeEffectiveness['normal']['fighting'] = 1.6;
  typeEffectiveness['normal']['ghost'] = 0.39;
  typeEffectiveness['fighting']['flying'] = 1.6;
  typeEffectiveness['fighting']['psychic'] = 1.6;
  typeEffectiveness['fighting']['fairy'] = 1.6;
  typeEffectiveness['fighting']['rock'] = 0.63;
  typeEffectiveness['fighting']['bug'] = 0.63;
  typeEffectiveness['fighting']['dark'] = 0.63;
  typeEffectiveness['flying']['rock'] = 1.6;
  typeEffectiveness['flying']['electric'] = 1.6;
  typeEffectiveness['flying']['ice'] = 1.6;
  typeEffectiveness['flying']['fighting'] = 0.63;
  typeEffectiveness['flying']['grass'] = 0.63;
  typeEffectiveness['flying']['bug'] = 0.63;
  typeEffectiveness['flying']['ground'] = 0.39;
  typeEffectiveness['poison']['ground'] = 1.6;
  typeEffectiveness['poison']['psychic'] = 1.6;
  typeEffectiveness['poison']['fighting'] = 0.63;
  typeEffectiveness['poison']['poison'] = 0.63;
  typeEffectiveness['poison']['bug'] = 0.63;
  typeEffectiveness['poison']['grass'] = 0.63;
  typeEffectiveness['poison']['fairy'] = 0.63;
  typeEffectiveness['ground']['water'] = 1.6;
  typeEffectiveness['ground']['grass'] = 1.6;
  typeEffectiveness['ground']['ice'] = 1.6;
  typeEffectiveness['ground']['poison'] = 0.63;
  typeEffectiveness['ground']['rock'] = 0.63;
  typeEffectiveness['ground']['electric'] = 0.39;
  typeEffectiveness['rock']['water'] = 1.6;
  typeEffectiveness['rock']['grass'] = 1.6;
  typeEffectiveness['rock']['fighting'] = 1.6;
  typeEffectiveness['rock']['ground'] = 1.6;
  typeEffectiveness['rock']['steel'] = 1.6;
  typeEffectiveness['rock']['normal'] = 0.63;
  typeEffectiveness['rock']['poison'] = 0.63;
  typeEffectiveness['rock']['fire'] = 0.63;
  typeEffectiveness['rock']['flying'] = 0.63;
  typeEffectiveness['bug']['fire'] = 1.6;
  typeEffectiveness['bug']['flying'] = 1.6;
  typeEffectiveness['bug']['rock'] = 1.6;
  typeEffectiveness['bug']['grass'] = 0.63;
  typeEffectiveness['bug']['fighting'] = 0.63;
  typeEffectiveness['bug']['ground'] = 0.63;
  typeEffectiveness['ghost']['ghost'] = 1.6;
  typeEffectiveness['ghost']['dark'] = 1.6;
  typeEffectiveness['ghost']['poison'] = 0.63;
  typeEffectiveness['ghost']['bug'] = 0.63;
  typeEffectiveness['ghost']['normal'] = 0.39;
  typeEffectiveness['ghost']['fighting'] = 0.39;
  typeEffectiveness['steel']['fire'] = 1.6;
  typeEffectiveness['steel']['fighting'] = 1.6;
  typeEffectiveness['steel']['ground'] = 1.6;
  typeEffectiveness['steel']['normal'] = 0.63;
  typeEffectiveness['steel']['grass'] = 0.63;
  typeEffectiveness['steel']['ice'] = 0.63;
  typeEffectiveness['steel']['flying'] = 0.63;
  typeEffectiveness['steel']['psychic'] = 0.63;
  typeEffectiveness['steel']['bug'] = 0.63;
  typeEffectiveness['steel']['rock'] = 0.63;
  typeEffectiveness['steel']['dragon'] = 0.63;
  typeEffectiveness['steel']['steel'] = 0.63;
  typeEffectiveness['steel']['fairy'] = 0.63;
  typeEffectiveness['fire']['ground'] = 1.6;
  typeEffectiveness['fire']['rock'] = 1.6;
  typeEffectiveness['fire']['water'] = 1.6;
  typeEffectiveness['fire']['fire'] = 0.63;
  typeEffectiveness['fire']['grass'] = 0.63;
  typeEffectiveness['fire']['ice'] = 0.63;
  typeEffectiveness['fire']['bug'] = 0.63;
  typeEffectiveness['fire']['steel'] = 0.63;
  typeEffectiveness['fire']['fairy'] = 0.63;
  typeEffectiveness['water']['electric'] = 1.6;
  typeEffectiveness['water']['grass'] = 1.6;
  typeEffectiveness['water']['fire'] = 0.63;
  typeEffectiveness['water']['water'] = 0.63;
  typeEffectiveness['water']['ice'] = 0.63;
  typeEffectiveness['water']['steel'] = 0.63;
  typeEffectiveness['grass']['fire'] = 1.6;
  typeEffectiveness['grass']['ice'] = 1.6;
  typeEffectiveness['grass']['poison'] = 1.6;
  typeEffectiveness['grass']['flying'] = 1.6;
  typeEffectiveness['grass']['bug'] = 1.6;
  typeEffectiveness['grass']['water'] = 0.63;
  typeEffectiveness['grass']['electric'] = 0.63;
  typeEffectiveness['grass']['grass'] = 0.63;
  typeEffectiveness['grass']['ground'] = 0.63;
  typeEffectiveness['electric']['ground'] = 1.6;
  typeEffectiveness['electric']['electric'] = 0.63;
  typeEffectiveness['electric']['flying'] = 0.63;
  typeEffectiveness['electric']['steel'] = 0.63;
  typeEffectiveness['psychic']['bug'] = 1.6;
  typeEffectiveness['psychic']['ghost'] = 1.6;
  typeEffectiveness['psychic']['dark'] = 1.6;
  typeEffectiveness['psychic']['fighting'] = 0.63;
  typeEffectiveness['psychic']['psychic'] = 0.63;
  typeEffectiveness['ice']['fire'] = 1.6;
  typeEffectiveness['ice']['fighting'] = 1.6;
  typeEffectiveness['ice']['rock'] = 1.6;
  typeEffectiveness['ice']['steel'] = 1.6;
  typeEffectiveness['ice']['ice'] = 0.63;
  typeEffectiveness['dragon']['ice'] = 1.6;
  typeEffectiveness['dragon']['dragon'] = 1.6;
  typeEffectiveness['dragon']['fairy'] = 1.6;
  typeEffectiveness['dragon']['fire'] = 0.63;
  typeEffectiveness['dragon']['water'] = 0.63;
  typeEffectiveness['dragon']['electric'] = 0.63;
  typeEffectiveness['dragon']['grass'] = 0.63;
  typeEffectiveness['dark']['fighting'] = 1.6;
  typeEffectiveness['dark']['bug'] = 1.6;
  typeEffectiveness['dark']['fairy'] = 1.6;
  typeEffectiveness['dark']['ghost'] = 0.63;
  typeEffectiveness['dark']['dark'] = 0.63;
  typeEffectiveness['dark']['psychic'] = 0.39;
  typeEffectiveness['fairy']['poison'] = 1.6;
  typeEffectiveness['fairy']['steel'] = 1.6;
  typeEffectiveness['fairy']['fighting'] = 0.63;
  typeEffectiveness['fairy']['bug'] = 0.63;
  typeEffectiveness['fairy']['dark'] = 0.63;
  typeEffectiveness['fairy']['dragon'] = 0.39;
  return typeEffectiveness[type1][type2];
};

const calculateCombinedEffectiveness = (type1, type2) => {
  const combinedEffectivenessMap = new Map();

  types.forEach((attacker) => {
    const effectiveness1 = getTypeEffectiveness(
      type1,
      attacker,
    );
    const effectiveness2 = getTypeEffectiveness(
      type2,
      attacker,
    );
    const combinedEffectiveness =
      effectiveness1 * effectiveness2;

    combinedEffectivenessMap.set(
      attacker,
      combinedEffectiveness,
    );
  });

  // Return the Map
  return combinedEffectivenessMap;
};

select('#reset-button').on('click', () => {
  selectedType = null;
  render(visualization);
});

csv('pokemon_1500.csv').then((loadedData) => {
  data = loadedData;
  filteredData = loadedData;
  /*
  types = Array.from(
    new Set(
      data.flatMap((d) => [d['Type 1'], d['Type 2']]),
    ),
  );*/
  types = Object.keys(allTypes);
  types.forEach((type) => {
    if (type != 'none') {
      typeCounts[type] = data.filter(
        (d) => d['Type 1'] === type || d['Type 2'] === type,
      ).length;
    }
  });
  //console.log(data);
  select('#visualization-menu').call(dropdownMenu, {
    options: [
      'main',
      'fast moves',
      'charge moves',
      //'scatter',
    ],
    onOptionClicked: onVisualizationClicked,
    selectedOption: visualization,
  });
  render(visualization);
});

csv('fast_moves.csv').then((loadedData) => {
  fastMovesData = loadedData;
  fastMovesData.forEach((d) => {
    d['DPT'] = +d['DPT'];
    d['EPT'] = +d['EPT'];
  });
  fastMovesData = fastMovesData.filter(
    (d) =>
      (d.Moves != 'Yawn') &
      (d.Moves != 'Splash') &
      (d.Moves != 'Transform'),
  );
});

csv('charge_moves.csv').then((loadedData) => {
  chargeMovesData = loadedData;
  chargeMovesData.forEach((d) => {
    d['Base Power'] = +d['Base Power'];
    d['Energy'] = Math.abs(d['Energy']);
  });
  console.log(chargeMovesData);
})